<?php
/**
 * Plugin Name: DEV PHP Error Log
 * Plugin URI: https://www.easytoolswp.com/
 * Description: Enable or disable PHP error logging and manage error logs.
 * Version: 1.2.2
 * Author: EasyToolsWP.com
 * Author URI: https://www.easytoolswp.com/
 * Update URI: https://www.easytoolswp.com/plugins/dev/php-log-error-etwp.zip
 */

add_action('admin_menu', 'php_error_logger_menu');
add_action('admin_init', 'save_plugin_settings');
add_action('wp_dashboard_setup', 'add_php_error_log_widget');

function php_error_logger_menu() {
    add_submenu_page(
        'tools.php',
        'PHP Error Log',
        'PHP Error Log',
        'manage_options',
        'php-error-log',
        'ms_webmaster_error_log_page'
    );
}

function ms_webmaster_error_log_page() {
    if (isset($_POST['save_settings'])) {
        save_plugin_settings();
        echo '<div class="notice notice-success is-dismissible">';
        echo '<p><strong>Saved!</strong> Settings have been successfully saved.</p>';
        echo '</div>';
    }

    if (isset($_POST['clear_log'])) {
        clear_error_log();
        echo '<div class="notice notice-success is-dismissible">';
        echo '<p><strong>Log Cleared!</strong> The error log has been cleared.</p>';
        echo '</div>';
    }

    echo '<div class="wrap">';
    echo '<div class="notice notice-warning">';
    echo '<p><strong>Warning:</strong> Changing settings may affect your site. Be cautious.</p>';
    echo '</div>';

    // Settings form
    echo '<form method="post" action="">';
    $log_errors = get_option('log_errors', 0);
    $display_errors = get_option('display_errors', 0);
    $error_log_path = get_option('error_log_path', plugin_dir_path(__FILE__) . 'PHP-error.log');
    $lines_to_display = get_option('lines_to_display', 10);

    echo '<label for="log_errors">Enable PHP Error Logging:</label>';
    echo '<input type="checkbox" id="log_errors" name="log_errors" value="1" ' . checked(1, $log_errors, false) . '>';
    echo '<br><br>';
    echo '<label for="display_errors">Display Errors Publicly:</label>';
    echo '<input type="checkbox" id="display_errors" name="display_errors" value="1" ' . checked(1, $display_errors, false) . '>';
    echo '<br><br>';
    echo '<label for="error_log_path">Path to Log File:</label>';
    echo '<input type="text" id="error_log_path" name="error_log_path" value="' . esc_attr($error_log_path) . '" readonly>';
    echo '<br><br>';
    
    echo '<label for="lines_to_display">Lines to Display:</label>';
    echo '<select id="lines_to_display" name="lines_to_display">';
    $options = array(10, 20, 30, 50);
    foreach ($options as $option) {
        echo '<option value="' . esc_attr($option) . '" ' . selected($option, $lines_to_display, false) . '>' . esc_html($option) . '</option>';
    }
    echo '</select>';
    echo '<br><br>';
    echo '<input type="submit" name="save_settings" class="button button-primary" value="Save Settings">';
    echo '</form>';

    if ($log_errors) {
        $log_content = get_error_log_content($lines_to_display);
        echo '<textarea rows="20" cols="80" readonly>' . esc_textarea($log_content) . '</textarea>';

        // Clear log form
        echo '<form method="post" action="">';
        echo '<input type="submit" name="clear_log" class="button button-secondary" value="Clear Log" onclick="return confirm(\'Are you sure you want to delete the log?\');">';
        echo '</form>';
    } else {
        echo '<p>Error logging is currently disabled. Enable it below.</p>';
    }

    echo '</div>';
}

function get_error_log_content($lines_to_display) {
    $error_log_path = get_option('error_log_path', plugin_dir_path(__FILE__) . 'PHP-error.log');
    
    if (!file_exists($error_log_path)) {
        @file_put_contents($error_log_path, '');
        @chmod($error_log_path, 0666);
    }

    if (file_exists($error_log_path)) {
        $error_log_content = file($error_log_path, FILE_IGNORE_NEW_LINES);
        $error_log_content = array_slice($error_log_content, -$lines_to_display, $lines_to_display, true);
        return implode("\n", $error_log_content);
    } else {
        return ''; // Return an empty string if log file not found
    }
}

function clear_error_log() {
    $error_log_path = get_option('error_log_path', plugin_dir_path(__FILE__) . 'PHP-error.log');
    @file_put_contents($error_log_path, ''); // Clear the log
    @chmod($error_log_path, 0666);
}

function save_plugin_settings() {
    if (isset($_POST['save_settings'])) {
        $log_errors = isset($_POST['log_errors']) ? 1 : 0;
        update_option('log_errors', $log_errors);
        update_option('display_errors', isset($_POST['display_errors']) ? 1 : 0);
        update_option('error_log_path', plugin_dir_path(__FILE__) . 'PHP-error.log');
        update_option('lines_to_display', intval($_POST['lines_to_display']));

        // Tilføj eller fjern fejllogning i functions.php baseret på indstillingen
        if ($log_errors) {
            toggle_error_logging('add');
        } else {
            toggle_error_logging('remove');
        }
    }
}

function toggle_error_logging($action = 'add') {
    $functions_file = get_template_directory() . '/functions.php';
    $error_log_path = get_option('error_log_path', plugin_dir_path(__FILE__) . 'PHP-error.log');

    $log_code = <<<EOT

// Enable error logging.
@ini_set('log_errors', 'On');
@ini_set('error_log', '$error_log_path');

// Don't show errors to site visitors.
@ini_set('display_errors', 'Off');
if ( !defined('WP_DEBUG_DISPLAY') ) {
    define('WP_DEBUG_DISPLAY', false);
}

EOT;

    // Læs indholdet af functions.php-filen
    $file_content = file_get_contents($functions_file);

    if ($action === 'add') {
        // Tjek om koden allerede er til stede
        if (strpos($file_content, '@ini_set') === false) {
            // Tilføj koden til slutningen af filen
            file_put_contents($functions_file, $file_content . $log_code);
        }
    } elseif ($action === 'remove') {
        // Fjern koden, hvis den findes
        $updated_content = str_replace($log_code, '', $file_content);
        file_put_contents($functions_file, $updated_content);
    }
}

function add_php_error_log_widget() {
    $log_errors = get_option('log_errors', 0);

    if ($log_errors) {
        wp_add_dashboard_widget(
            'php_error_log_widget',
            'PHP Error Log',
            'display_php_error_log_widget'
        );
    }
}

function display_php_error_log_widget() {
    $log_errors = get_option('log_errors', 0);
    $lines_to_display = get_option('lines_to_display', 10);
    $log_content = get_error_log_content($lines_to_display);

    // Check if logging is enabled before displaying the widget
    if (!$log_errors) {
        echo '<p>Error logging is currently disabled. <a href="admin.php?page=php-error-log">Enable it in the settings</a>.</p>';
        return;
    }

    echo '<div style="max-height: 300px; overflow-y: auto; border: 1px solid #ddd; padding: 10px;">';
    echo '<p><strong>Recent PHP Errors:</strong></p>';

    if (!empty($log_content)) {
        $lines = explode("\n", $log_content);
        foreach ($lines as $line) {
            if (empty($line)) continue;

            // Parse the log line
            if (preg_match('/\[(.*?)\] (.*? error):(.*)/i', $line, $matches)) {
                $date = $matches[1];
                $error_type = ucfirst(trim($matches[2]));
                $error_message = trim($matches[3]);

                // Apply color coding based on error type
                $color = '#ff6f6f'; // Default color for errors
                if (stripos($error_type, 'warning') !== false) {
                    $color = '#ffc107';
                } elseif (stripos($error_type, 'notice') !== false) {
                    $color = '#17a2b8';
                }

                // Display the formatted log
                echo "<div style='margin-bottom: 10px; padding: 5px; border-left: 4px solid $color;'>";
                echo "<strong>Date:</strong> <span style='color: #888;'>$date</span><br>";
                echo "<strong>Type:</strong> <span style='color: $color;'>$error_type</span><br>";
                echo "<strong>Message:</strong> $error_message";

                // Check if the line contains a stack trace (lines that are indented)
                if (stripos($error_message, 'Stack trace:') !== false || preg_match('/^\s+/', $line)) {
                    echo "<pre style='margin-top: 5px; background: #f1f1f1; padding: 5px; border-radius: 3px; font-size: 12px; color: #666; white-space: pre-wrap;'>";
                    echo "<code>" . htmlspecialchars($line) . "</code>";
                    echo '</pre>';
                }

                // Copy button for the entire log entry
                echo '<button onclick="copyToClipboard(this)" style="margin-top: 5px; padding: 5px 10px; background-color: #0073aa; color: white; border: none; border-radius: 3px; cursor: pointer;">Copy</button>';

                echo "</div>";
            } else {
                // If the line doesn't match the expected format, print it raw
                echo "<div style='margin-bottom: 10px; padding: 5px;'>";
                echo "<pre>" . htmlspecialchars($line) . "</pre>";
                echo '</div>';
            }
        }
    } else {
        echo '<p>No errors logged.</p>';
    }

    // Add a clear log button and a link to settings
    echo '<div style="margin-top: 10px;">';
    echo '<button onclick="clearErrorLog()" style="padding: 5px 10px; background-color: #d63638; color: white; border: none; border-radius: 3px; cursor: pointer;">Clear Log</button>';
    echo '<a href="admin.php?page=php-error-log" style="margin-left: 10px; color: #0073aa;">Go to Settings</a>';
    echo '</div>';
    echo '</div>';
    ?>

    <script>
        function copyToClipboard(button) {
            var codeBlock = button.previousElementSibling;
            var range = document.createRange();
            range.selectNodeContents(codeBlock);
            var selection = window.getSelection();
            selection.removeAllRanges();
            selection.addRange(range);
            document.execCommand('copy');
            alert('Log entry copied to clipboard!');
        }

        function clearErrorLog() {
            if (confirm('Are you sure you want to clear the log?')) {
                // Make an AJAX request to clear the log
                jQuery.post(ajaxurl, { action: 'clear_error_log' }, function(response) {
                    alert('Log cleared!');
                    location.reload(); // Reload the widget to show updated content
                });
            }
        }
    </script>
    <?php
}

// Handle AJAX request to clear the log
add_action('wp_ajax_clear_error_log', 'clear_error_log');
